// digt.h
// module for computing the DIGT value
// copyright SafeTP Development Group, Inc., 2000  Terms of use are as specified in license.txt

#ifndef __DIGT_H
#define __DIGT_H

#include "typ.h"        // byte, bool
#include "datablok.h"   // DataBlock

class SHA;              // sha.h
class Request;          // request.h
class Reply;            // reply.h

class DigestComputer {
  SHA *sha;                   // SHA computation object
  DataBlock computedDigt;     // digest if already computed
  bool alreadyComputed;       // true if 'computedDigt' is correct

public:     // data
  static bool echoDigestInput;
    // when true, we print everything folded into the computation
    // default: false

public:     // funcs
  DigestComputer();
  ~DigestComputer();

  // fold a request or reply into the calculation
  // (illegal once the digest has been retrieved)
  void add(Request const &request);
  void add(Reply const &reply);

  // useful if the request or reply is just in a buffer (particularly
  // since Reply does *not* preserve the precise contents of
  // what is sent over the wire)
  void add(byte const *buffer, int length);

  // retrieve the digest (this can be called multiple
  // times, but once called, add() cannot be called again)
  DataBlock getDigt();
};

#endif // __DIGT_H

