// sftpcdoc.h
// declarations for sftpcdoc.cpp

#ifndef __SFTPCDOC_H
#define __SFTPCDOC_H

#include "typ.h"       // bool

// define an alias for a command
struct Alias {
  char const *alias;           // command user enters
  char const *expansion;       // string into which it expands
};

// all aliases
extern Alias const aliases[];
extern int const numAliases;

// return index of alias, or -1 if not found
int findAlias(char const *alias);


// documentation for a (non-alias) command
struct Command {
  bool common;           // true for commands listed in 'help' command
  char const *name;      // name as typed by user
  char const *args;      // arguments format
  char const *blurb;     // one-line summary
  char const *arglines;  // LF-separated lines for each argument
  char const *desc;      // LF-separated lines describing cmd generally
};

// docs for all command
extern Command const commands[];
extern int const numCommands;

// return index of command, or -1 if not found
int findCommand(char const *name);


// print list of commands (either the common ones, or the
// uncommon ones)
void printCommandList(bool common);


// write part of the HTML documentation to stdout
void writeHTMLCommands();


#endif // __SFTPCDOC_H

